/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation version
 * 2.1 of the License.
 *                                                                                  
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *                                                                                  
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * END COPYRIGHT BLOCK **/

#include "libadminutil/resource.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "unicode/ures.h"
#include "unicode/ustring.h"


/*
  ----------------------------------------------------------------
  res_init_resource

  Initializes a property file path.
  -----------------------------------------------------------------
*/
PR_IMPLEMENT(Resource*)
res_init_resource(const char* path, const char* package)
{
  Resource *resource;
  char *resPath;
  char path_last_char;

  if (package == NULL || PL_strlen(package) == 0) {
    return NULL;
  }

  if (path == NULL) {
      path = "./";
  }

  path_last_char = path[PL_strlen(path) - 1];
  if (path_last_char != '/' && path_last_char != '\\') {
    resPath = PR_smprintf("%s/%s", path, package);
  } else {
    resPath = PR_smprintf("%s%s", path, package);
  }
  
  resource = (Resource *)PR_Malloc(sizeof(Resource));
  if (NULL == resource) {
    return NULL;
  }
  memset(resource, 0, sizeof(Resource));

  /* ICU on Win does not like forward slashes in paths */
#ifdef XP_WIN32
  {
      char *c;
      for (c=resPath; *c; c++) {
          if (*c == '/') *c = '\\';
      }
  }
#endif

  resource->path = resPath;
  return resource;
}

/*
  -----------------------------------------------------------------------------
  res_getstring

  Gets a string by key from a resource file
  -----------------------------------------------------------------------------
*/
PR_IMPLEMENT(const char*)
res_getstring(Resource* resource, char *key, char *locale)
{
  const char* result = NULL;
  UResourceBundle *bundle = NULL;
  UErrorCode status = U_ZERO_ERROR;
  

  if (resource == NULL || resource->path == NULL || key == NULL) {
      return PL_strdup("");
  }

  bundle = ures_open(resource->path, (const char*)locale, &status);

  if(U_SUCCESS(status) && bundle) {
      int32_t umsglen=0;
      const UChar *umsg = ures_getStringByKey(bundle, key, &umsglen, &status);

      if (U_SUCCESS(status) && umsg) {
             int32_t msglen=-1;

         /* Get first the required buffer size */
         u_strToUTF8(NULL, 0, &msglen, umsg, umsglen, &status);
         if (msglen >0) {				 
             result = PR_Malloc(msglen+1);
             if (result) {
                  /* reset status, set to OVERFLOW by the last call to u_strToUTF8 */
                  status = U_ZERO_ERROR; 
                  /* now the real conversion with allocated buffer */
                  u_strToUTF8((char*)result, msglen+1, &msglen, umsg, umsglen, &status);
                  if (!U_SUCCESS(status)) {
                      result = NULL;
                  }
             }
         }
      }

      ures_close(bundle);
  }

  if (result == NULL) {
      result = PL_strdup("");
  }

  return result;
}


/*
   ----------------------------------------------------- 
   res_destroy_resource

   Frees resource returned by the init routine
   -----------------------------------------------------
 */

PR_IMPLEMENT(void)
res_destroy_resource(Resource* resource)
{
  if(resource == NULL) {
    return;
  }
  if(resource->path != NULL) {
    PR_Free(resource->path);
  }
  if(resource->package != NULL) {
    PR_Free(resource->package);
  }
  PR_Free(resource);
}
